@@ -30,6 +30,8 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <time.h>
+#include <utime.h>
 #include <fcntl.h>
 #include <strings.h>
 #include <termios.h>
@@ -55,27 +57,22 @@
 static gboolean file_is_dir(const char *filename) {
     struct stat st;
 
-    if (stat(filename, &st) < 0) {
+    if (stat(filename, &st) < 0)
         return FALSE;
-    }
 
-    if (S_ISDIR(st.st_mode)) {
+    if (S_ISDIR(st.st_mode))
         return TRUE;
-    }
 
     return FALSE;
 }
 
 static void gw_obex_set_error(GwObex *ctx) {
-    if (ctx->abort) {
+    if (ctx->abort)
         ctx->error = GW_OBEX_ERROR_ABORT;
-    }
-    else if (ctx->conn_fd < 0 || ctx->link_err) {
+    else if (ctx->conn_fd < 0 || ctx->link_err)
         ctx->error = GW_OBEX_ERROR_DISCONNECT;
-    }
-    else {
+    else
         ctx->error = (gint)ctx->obex_rsp;
-    }
 }
 
 static void idle_callback(obex_t *handle, obex_object_t *object, int mode,
@@ -90,7 +87,7 @@ static void obex_link_error(GwObex *ctx) {
     ctx->link_err = TRUE;
     OBEX_SetUserCallBack(ctx->handle, idle_callback, NULL);
     ctx->done = TRUE;
-    ctx->conid = -1;
+    ctx->conid = CONID_INVALID;
     if (ctx->conn_fd >= 0) {
         OBEX_TransportDisconnect(ctx->handle);
         close(ctx->conn_fd);
@@ -114,17 +111,18 @@ static void obex_link_error(GwObex *ctx) {
 }
 
 static gboolean gw_obex_do_abort(GwObex *ctx) {
-    if (ctx->conn_fd < 0) {
+    debug("gw_obex_do_abort()\n");
+
+    if (ctx->conn_fd < 0)
         return FALSE;
-    }
 
     ctx->abort = TRUE;
 
 #ifdef USE_NICE_ABORT
     debug("Performing nice abort\n");
-    if (OBEX_CancelRequest(ctx->handle, TRUE) != 0) {
+    if (OBEX_CancelRequest(ctx->handle, TRUE) != 0)
         return FALSE;
-    }
+    ctx->abort_sent = TRUE;
     return TRUE;
 #else
     debug("Performing abort through disconnection (without ABORT command)\n");
@@ -137,9 +135,10 @@ static gboolean gw_obex_do_abort(GwObex *ctx) {
 
 static gboolean gw_obex_request_sync(GwObex *ctx, obex_object_t *object) {
     /* Set sensible start values */
-    ctx->done    = FALSE;
-    ctx->abort   = FALSE;
-    ctx->counter = 0;
+    ctx->done       = FALSE;
+    ctx->abort      = FALSE;
+    ctx->abort_sent = FALSE;
+    ctx->counter    = 0;
 
     if (OBEX_Request(ctx->handle, object) < 0) {
         debug("OBEX_Request() failed\n");
@@ -150,16 +149,17 @@ static gboolean gw_obex_request_sync(GwObex *ctx, obex_object_t *object) {
     while (TRUE) {
         int ret;
 
-        ret = OBEX_HandleInput(ctx->handle, 30);
+        ret = OBEX_HandleInput(ctx->handle, 10);
 
         if (ctx->done)
             break;
 
         if (ctx->cancel_cb && ctx->cancel_cb(ctx->cancel_data)) {
             debug("cancel_cb() returned TRUE, aborting.\n");
-            if (!gw_obex_do_abort(ctx)) {
+            if (ctx->abort_sent)
+                continue;
+            if (!gw_obex_do_abort(ctx))
                 break;
-            }
             /* Must continue to receive the abort reply */
             continue;
         }
@@ -172,6 +172,7 @@ static gboolean gw_obex_request_sync(GwObex *ctx, obex_object_t *object) {
         else if (ret == 0) { /* Timeout */
             debug("OBEX_HandleInput(): timeout\n");
             ctx->error = GW_OBEX_ERROR_TIMEOUT;
+            obex_link_error(ctx);
             return FALSE;
         }
         
@@ -180,8 +181,13 @@ static gboolean gw_obex_request_sync(GwObex *ctx, obex_object_t *object) {
 
     gw_obex_set_error(ctx);
 
-    if (ctx->error == OBEX_RSP_SUCCESS)
+    if (ctx->error == OBEX_RSP_SUCCESS) {
+        /* It is possible that a EV_PROGRESS doesn't arrive after all data has
+         * been transfered. Call pr_cb here to ensure app gets 100% progress */
+        if (ctx->report_progress && ctx->pr_cb)
+            ctx->pr_cb(ctx, ctx->obex_op, ctx->counter, ctx->counter, ctx->pr_data);
         return TRUE;
+    }
     else
         return FALSE;
 }
@@ -227,7 +233,7 @@ static void obex_connect_done(GwObex *ctx, obex_object_t *object, int obex_rsp)
         int new_size;
         debug("Version: 0x%02x. Flags: 0x%02x  OBEX packet length: %d\n",
                 nonhdrdata->version, nonhdrdata->flags, mtu);
-        new_size = mtu - 100;
+        new_size = mtu - 200;
         /*OBEX_SetTransportMTU(ctx->handle, mtu, mtu);*/
         if (new_size < ctx->chunk_size) {
             debug("Resizing stream chunks to %d\n", new_size);
@@ -263,8 +269,10 @@ static void show_headers(obex_t *handle, obex_object_t *object) {
     obex_headerdata_t hv;
     uint8_t hi;
     unsigned int hlen;
+
     while (OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, &hlen)) {
         char *str;
+
         switch (hi) {
             case OBEX_HDR_WHO:
                 debug("OBEX_HDR_WHO\n");
@@ -272,32 +280,51 @@ static void show_headers(obex_t *handle, obex_object_t *object) {
             case OBEX_HDR_CONNECTION:
                 debug("OBEX_HDR_CONNECTION: %#x\n", hv.bq4);
                 break;
+            case OBEX_HDR_LENGTH:
+                debug("OBEX_HDR_LENGTH: %d\n", hv.bq4);
+                break;
+            case OBEX_HDR_NAME:
+                str = g_utf16_to_utf8((gunichar2 *)hv.bs, hlen, NULL, NULL, NULL);
+                if (str) {
+                    debug("OBEX_HDR_NAME: %s\n", str);
+                    g_free(str);
+                }
+                break;
             case OBEX_HDR_AUTHCHAL:
                 str = bytestr(hv.bs, hlen);
                 debug("OBEX_HDR_AUTHCHAL: %s\n", str);
                 g_free(str);
                 break;
+            case OBEX_HDR_TIME:
+                str = g_strdup_printf("OBEX_HDR_TIME: %%%us\n", hlen);
+                debug(str, hv.bs);
+                g_free(str);
+                break;
+            case OBEX_HDR_TYPE:
+                debug("OBEX_HDR_TYPE: %s\n", hv.bs);
+                break;
             default:
                 debug("Skipped header 0x%02x\n", hi);
                 break;
         }
     }
+
+    OBEX_ObjectReParseHeaders(handle, object);
 }
 #endif
 
 static void obex_abort_done(GwObex *ctx, obex_object_t *object,
                             int obex_cmd, int obex_rsp) {
     ctx->done = TRUE;
+    ctx->abort_sent = FALSE;
 
-    if (obex_rsp != OBEX_RSP_SUCCESS) {
+    if (obex_rsp != OBEX_RSP_SUCCESS)
         debug("ABORT of %s command (0x%02x) failed: %s (0x%02x)\n",
                 optostr((uint8_t)obex_cmd), (uint8_t)obex_cmd,
                 OBEX_ResponseToString(obex_rsp), (uint8_t)obex_rsp);
-    }
-    else {
+    else
         debug("ABORT of %s command (0x%02x) succeeded.\n",
                 optostr((uint8_t)obex_cmd), (uint8_t)obex_cmd);
-    }
 }
 
 static void obex_request_done(GwObex *ctx, obex_object_t *object,
@@ -330,52 +357,57 @@ static void obex_request_done(GwObex *ctx, obex_object_t *object,
     }
 }
 
-static gint get_target_size(obex_t *handle, obex_object_t *object) {
+static void get_target_size_and_time(obex_t *handle, obex_object_t *object,
+                                     gint *size, time_t *time) {
     obex_headerdata_t hv;
     uint8_t hi;
     unsigned int hlen;
-    gint size = GW_OBEX_UNKNOWN_LENGTH;
+
+    *size = GW_OBEX_UNKNOWN_LENGTH;
+    *time = -1;
+
     while (OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, &hlen)) {
         switch (hi) {
             case OBEX_HDR_LENGTH:
-                size = hv.bq4; //(gint) g_ntohl(hv.bq4);
-                debug("Target size: %d\n", size);
+                *size = hv.bq4; //(gint) g_ntohl(hv.bq4);
+                break;
+            case OBEX_HDR_TIME:
+                *time = parse_iso8601((char *)hv.bs, hlen);
                 break;
             default:
                 break;
         }
     }
+
     OBEX_ObjectReParseHeaders(handle, object);
-    return size;
 }
 
 static void obex_readstream(GwObex *ctx, obex_object_t *object) {
     const uint8_t *buf;
     int actual;
 
-    /* Get the target size if necessary and possible */
-    if (ctx->counter == 0 && ctx->target_size == GW_OBEX_UNKNOWN_LENGTH) {
-        ctx->target_size = get_target_size(ctx->handle, object);
-    }
+    if (ctx->counter == 0)
+        get_target_size_and_time(ctx->handle, object,
+                                 &ctx->target_size, &ctx->modtime);
+
+#ifdef DEBUG
+    if (ctx->counter == 0)
+        show_headers(ctx->handle, object);
+#endif
     
     actual = OBEX_ObjectReadStream(ctx->handle, object, &buf);
     if (actual > 0) {
         ctx->counter += actual;
-        if (ctx->stream_fd >= 0) {
+        if (ctx->stream_fd >= 0)
             (void) write(ctx->stream_fd, buf, actual);
-        }
         else {
             ctx->buf = g_realloc(ctx->buf, ctx->counter);
             memcpy(&ctx->buf[ctx->buf_size], buf, actual);
             ctx->buf_size = ctx->counter;
         }
-        if (ctx->pr_cb) {
-            ctx->pr_cb(ctx, ctx->obex_op, ctx->counter, ctx->target_size, ctx->pr_data);
-        }
     }
-    else {
+    else
         debug("Error or no data on OBEX stream\n");
-    }
 }
 
 static void obex_writestream(GwObex *ctx, obex_object_t *object) {
@@ -389,13 +421,13 @@ static void obex_writestream(GwObex *ctx, obex_object_t *object) {
         if (ctx->counter > 4000)
             actual = -1;
 #endif
-        if (actual > 0) {
+        if (actual > 0)
             OBEX_ObjectAddHeader(ctx->handle, object, OBEX_HDR_BODY,
                     hv, actual, OBEX_FL_STREAM_DATA);
-        } else if (actual == 0) { /* EOF */
+        else if (actual == 0) /* EOF */
             OBEX_ObjectAddHeader(ctx->handle, object, OBEX_HDR_BODY,
                     hv, 0, OBEX_FL_STREAM_DATAEND);
-        } else { /* error reading file */
+        else { /* error reading file */
             debug("read(): %s\n", strerror(errno));
             gw_obex_do_abort(ctx);
         }
@@ -417,13 +449,8 @@ static void obex_writestream(GwObex *ctx, obex_object_t *object) {
         }
     }
 
-    /* Update progress */
-    if (actual > 0) {
+    if (actual > 0)
         ctx->counter += actual;
-        if (ctx->pr_cb) {
-            ctx->pr_cb(ctx, ctx->obex_op, ctx->counter, ctx->target_size, ctx->pr_data);
-        }
-    }
 }
 
 static void obex_event_handler(obex_t *handle, obex_object_t *object, int mode,
@@ -436,6 +463,8 @@ static void obex_event_handler(obex_t *handle, obex_object_t *object, int mode,
             break;
         case OBEX_EV_PROGRESS:
             debug("OBEX_EV_PROGRESS\n");
+            if (ctx->report_progress && ctx->pr_cb)
+                ctx->pr_cb(ctx, ctx->obex_op, ctx->counter, ctx->target_size, ctx->pr_data);
             break;
         case OBEX_EV_REQDONE:
             debug("OBEX_EV_REQDONE\n");
@@ -508,9 +537,9 @@ gboolean gw_obex_transport_setup(int fd, obex_t **handle) {
         return FALSE;
     }
 
-    (void) OBEX_SetTransportMTU(*handle, OBEX_DEFAULT_MTU, DEFAULT_CHUNK_SIZE + 200);
+    (void) OBEX_SetTransportMTU(*handle, GW_OBEX_RX_MTU, GW_OBEX_TX_MTU);
 
-    if (FdOBEX_TransportSetup(*handle, fd, fd, DEFAULT_CHUNK_SIZE + 200) < 0) {
+    if (FdOBEX_TransportSetup(*handle, fd, fd, 0) < 0) {
         debug("FdOBEX_TransportSetup() failed\n");
         OBEX_Cleanup(*handle);
         return FALSE;
@@ -535,9 +564,8 @@ gboolean gw_obex_cb(GIOChannel *chan, GIOCondition cond, gpointer data) {
         debug("obex_cb() called, but connection is closed\n");
         obex_link_error(ctx);
         GW_OBEX_UNLOCK(ctx);
-        if (ctx->dc_cb) {
+        if (ctx->dc_cb)
             ctx->dc_cb(ctx, ctx->dc_data);
-        }
         return FALSE;
     }
 
@@ -560,9 +588,8 @@ gboolean gw_obex_cb(GIOChannel *chan, GIOCondition cond, gpointer data) {
         obex_link_error(ctx);
 
         GW_OBEX_UNLOCK(ctx);
-        if (ctx->dc_cb) {
+        if (ctx->dc_cb)
             ctx->dc_cb(ctx, ctx->dc_data);
-        }
         return FALSE;
     }
 
@@ -573,17 +600,21 @@ gboolean gw_obex_cb(GIOChannel *chan, GIOCondition cond, gpointer data) {
     return TRUE;
 }
 
-#if 0
-gboolean gw_obex_disconnect(obex_t *handle) {
+gboolean gw_obex_disconnect(GwObex *ctx) {
     obex_object_t *object;
 
-    object = OBEX_ObjectNew(handle, OBEX_CMD_DISCONNECT);
+    object = OBEX_ObjectNew(ctx->handle, OBEX_CMD_DISCONNECT);
 
-    return gw_obex_request_sync(handle, object);
+    if (ctx->conid != CONID_INVALID) {
+        obex_headerdata_t hv;
+        hv.bq4 = ctx->conid;
+        OBEX_ObjectAddHeader(ctx->handle, object, OBEX_HDR_CONNECTION, hv, 4, 0);
+    }
+
+    return gw_obex_request_sync(ctx, object);
 }
-#endif
 
-gboolean gw_obex_connect(GwObex *ctx, const unsigned char *target, size_t target_len) {
+gboolean gw_obex_connect(GwObex *ctx, const char *target, size_t target_len) {
     gboolean ret;
     obex_object_t *object;
 
@@ -592,7 +623,7 @@ gboolean gw_obex_connect(GwObex *ctx, const unsigned char *target, size_t target
     object = OBEX_ObjectNew(ctx->handle, OBEX_CMD_CONNECT);
     if (target) {
         obex_headerdata_t hv;
-        hv.bs = target;
+        hv.bs = (const unsigned char *)target;
         OBEX_ObjectAddHeader(ctx->handle, object, OBEX_HDR_TARGET, hv, target_len, OBEX_FL_FIT_ONE_PACKET);
     }
 
@@ -602,18 +633,19 @@ gboolean gw_obex_connect(GwObex *ctx, const unsigned char *target, size_t target
 }
 
 GwObex *make_context(obex_t *handle) {
-    GwObex *context = g_malloc(sizeof(GwObex));
+    GwObex *context;
 
-    memset(context, 0, sizeof(GwObex));
+    context = g_new0(GwObex, 1);
 
     context->handle      = handle;
     context->conn_fd     = OBEX_GetFD(handle);
-    context->conid       = -1;
+    context->conid       = CONID_INVALID;
     context->stream_fd   = -1;
-    context->chunk_size  = DEFAULT_CHUNK_SIZE;
+    context->chunk_size  = GW_OBEX_TX_MTU - 200;
     context->obex_op     = OBEX_CMD_NONE;
     context->obex_rsp    = OBEX_RSP_SUCCESS;
     context->target_size = GW_OBEX_UNKNOWN_LENGTH;
+    context->modtime     = -1;
 
     return context;
 }
@@ -631,6 +663,11 @@ gboolean gw_obex_action_op(GwObex *ctx, const gchar *src, const gchar *dst,
 
     object = OBEX_ObjectNew(ctx->handle, OBEX_CMD_ACTION);
 
+    if (ctx->conid != CONID_INVALID) {
+        hv.bq4 = ctx->conid;
+        OBEX_ObjectAddHeader(ctx->handle, object, OBEX_HDR_CONNECTION, hv, 4, 0);
+    }
+
     hv.bq1 = action;
     OBEX_ObjectAddHeader(ctx->handle, object, OBEX_HDR_ACTION_ID, hv, 1, 0);
 
@@ -687,14 +724,13 @@ gboolean gw_obex_setpath(GwObex *ctx, const gchar *path, int flags) {
         is_name = TRUE;
     }
 
-    if (flags & SETPATH_CREATE) {
+    if (flags & SETPATH_CREATE)
         nonhdrdata.flags &= ~0x02;
-    }
 
     object = OBEX_ObjectNew(ctx->handle, OBEX_CMD_SETPATH);
     OBEX_ObjectSetNonHdrData(object, (uint8_t*)&nonhdrdata, 2);
 
-    if (ctx->conid != -1) {
+    if (ctx->conid != CONID_INVALID) {
         hv.bq4 = ctx->conid;
         OBEX_ObjectAddHeader(ctx->handle, object, OBEX_HDR_CONNECTION, hv, 4, 0);
     }
@@ -737,7 +773,7 @@ gboolean gw_obex_get(GwObex *ctx,
 
     ctx->target_size = GW_OBEX_UNKNOWN_LENGTH;
 
-    if (ctx->conid != -1) {
+    if (ctx->conid != CONID_INVALID) {
         hv.bq4 = ctx->conid;
         OBEX_ObjectAddHeader(ctx->handle, object, OBEX_HDR_CONNECTION, hv, 4, 0);
     }
@@ -747,9 +783,8 @@ gboolean gw_obex_get(GwObex *ctx,
         OBEX_ObjectAddHeader(ctx->handle, object, OBEX_HDR_TYPE, hv, strlen(type) + 1, 0);
     }
 
-    if (ctx->stream_fd >= 0) {
+    if (ctx->stream_fd >= 0)
         user_fd = TRUE;
-    }
 
     if (remote) {
         gunichar2 *uname;
@@ -768,12 +803,10 @@ gboolean gw_obex_get(GwObex *ctx,
     if (local) {
         ctx->stream_fd = open(local, O_WRONLY | O_CREAT, 0600);
         if (ctx->stream_fd < 0) {
-            if (errno == ENOENT || errno == ENODEV) {
+            if (errno == ENOENT || errno == ENODEV)
                 ctx->error = GW_OBEX_ERROR_INVALID_PARAMS;
-            }
-            else {
+            else
                 ctx->error = GW_OBEX_ERROR_LOCAL_ACCESS;
-            }
             debug("open(%s): %s", local, strerror(errno));
             OBEX_ObjectDelete(ctx->handle, object);
             return FALSE;
@@ -786,6 +819,8 @@ gboolean gw_obex_get(GwObex *ctx,
         ctx->stream_fd = -1;
     }
 
+    ctx->report_progress = TRUE;
+
     OBEX_ObjectReadStream(ctx->handle, object, NULL);
 
 #ifdef DEBUG
@@ -807,8 +842,16 @@ gboolean gw_obex_get(GwObex *ctx,
         }
     }
     else {
-        if (local)
+        if (local) {
             debug("%s stored in %s\n", remote ? remote : type, local);
+            if (ctx->modtime != -1) {
+                struct utimbuf ubuf;
+                ubuf.actime = time(NULL);
+                ubuf.modtime = ctx->modtime;
+                if (utime(local, &ubuf) < 0)
+                    debug("utime(%s): %s\n", local, g_strerror(errno));
+            }
+        }
         if (buf) {
             *buf = (gchar *)ctx->buf;
             *buf_size = ctx->buf_size;
@@ -832,7 +875,9 @@ gboolean gw_obex_get(GwObex *ctx,
     }
 #endif
 
+    ctx->report_progress = FALSE;
     ctx->target_size = GW_OBEX_UNKNOWN_LENGTH;
+    ctx->modtime     = -1;
 
     ctx->obex_op = OBEX_CMD_NONE;
 
@@ -864,9 +909,10 @@ gboolean gw_obex_put(GwObex *ctx,
         }
     }
 
-    if (ctx->stream_fd >= 0) {
+    if (ctx->stream_fd >= 0)
         user_fd = TRUE;
-    }
+
+    ctx->report_progress = TRUE;
 
     if (local) {
         if (file_is_dir(local)) {
@@ -879,12 +925,10 @@ gboolean gw_obex_put(GwObex *ctx,
 
         ctx->stream_fd = open(local, O_RDONLY);
         if (ctx->stream_fd < 0) {
-            if (errno == ENOENT || errno == ENODEV) {
+            if (errno == ENOENT || errno == ENODEV)
                 ctx->error = GW_OBEX_ERROR_INVALID_PARAMS;
-            }
-            else {
+            else
                 ctx->error = GW_OBEX_ERROR_LOCAL_ACCESS;
-            }
             debug("open(%s): %s", local, strerror(errno));
             if (uname)
                 g_free(uname);
@@ -893,20 +937,20 @@ gboolean gw_obex_put(GwObex *ctx,
         g_assert(ctx->buf == NULL);
         ctx->buf = g_malloc(ctx->chunk_size);
         ctx->buf_size = ctx->chunk_size;
-        debug("Sending %s to %s, size: %d\n", local, remote ? remote : type,
-                get_fdsize(ctx->stream_fd));
+        debug("Sending %s to %s\n", local, remote ? remote : type);
     }
     else if (buf) {
         ctx->buf = (unsigned char *)buf;
         ctx->buf_size = buf_size;
         ctx->stream_fd = -1;
-        debug("Sending to %s, size: %d\n", remote ? remote : type, buf_size);
+        debug("Sending to %s\n", remote ? remote : type);
     }
     else if (user_fd) {
         ctx->buf = NULL;
         ctx->buf_size = 0;
     }
     else { /* Delete */
+        ctx->report_progress = FALSE;
         ctx->stream_fd = -1;
         ctx->buf = NULL;
         ctx->buf_size = 0;
@@ -915,7 +959,7 @@ gboolean gw_obex_put(GwObex *ctx,
 
     object = OBEX_ObjectNew(ctx->handle, OBEX_CMD_PUT);
 
-    if (ctx->conid != -1) { 
+    if (ctx->conid != CONID_INVALID) { 
         hv.bq4 = ctx->conid;
         OBEX_ObjectAddHeader(ctx->handle, object, OBEX_HDR_CONNECTION, hv, 4, 0);
     }
@@ -932,21 +976,41 @@ gboolean gw_obex_put(GwObex *ctx,
     }
 
     if (ctx->stream_fd >= 0 || buf) {
-        gint size;
-        if (buf) {
+        gint size = -1;
+        time_t time = -1;
+
+        if (buf)
             size = buf_size;
-        }
         else {
-            size = get_fdsize(ctx->stream_fd); 
+            struct stat stats;
+            if (fstat(ctx->stream_fd, &stats) == 0) {
+                size = stats.st_size;
+                time = stats.st_mtime;
+            }
         }
+
+        if (time >= 0) {
+            char tstr[17];
+            int len;
+
+            len = make_iso8601(time, tstr, sizeof(tstr));
+
+            if (len >= 0) {
+                debug("Adding time header: %s\n", tstr);
+                hv.bs = (unsigned char *)tstr;
+                OBEX_ObjectAddHeader(ctx->handle, object, OBEX_HDR_TIME, hv, len, 0);
+            }
+        }
+
         if (size >= 0) {
             ctx->target_size = size;
+            debug("Adding size header: %d\n", size);
             hv.bq4 = (uint32_t)size;
             OBEX_ObjectAddHeader(ctx->handle, object, OBEX_HDR_LENGTH, hv, 4, 0);
         }
-        else {
+        else
            ctx->target_size = GW_OBEX_UNKNOWN_LENGTH; 
-        }
+
         hv.bs = NULL;
         OBEX_ObjectAddHeader(ctx->handle, object, OBEX_HDR_BODY, hv, 0, OBEX_FL_STREAM_START);
     }
@@ -975,12 +1039,12 @@ gboolean gw_obex_put(GwObex *ctx,
             else
                 debug("transfer speed: %.2f kB/s\n", speed / 1024);
         }
-        else if (ctx->target_size > 0) {
+        else if (ctx->target_size > 0)
             debug("%d bytes transfered in less than one second\n", ctx->target_size);
-        }
     }
 #endif
 
+    ctx->report_progress = FALSE;
     ctx->buf = NULL;
     ctx->buf_size = 0;
     ctx->target_size = GW_OBEX_UNKNOWN_LENGTH;
